sum infra_com*
sum infra_electr

gen lyear=ln(year)
*replace infra_com_fixedline = . if infra_com_fixedline==0  /* 0-observations for DRC, Guinea, and Guinea-Bissau in most recent decade make no sense */

*generate logged variables
gen linfra_phoneline = ln(infra_com_fixedline)
gen linfra_mobile = ln(infra_com_mobile)
gen linfra_internet = ln(infra_com_internet)
gen linfra_electricity = ln(infra_electricaccess)
gen linfra_broadband = ln(infra_com_broadband)
gen linfra_secureinternet = ln(infra_com_secureinternet)

*generate interpolation variables
gen linfra_phoneline_ip = linfra_phoneline
gen linfra_mobile_ip = linfra_mobile
gen linfra_internet_ip = linfra_internet
gen linfra_electricity_ip = linfra_electricity
gen linfra_secureinternet_ip = linfra_secureinternet

*1st round prediction mobile
qui reg linfra_mobile linfra_phoneline i.geo i.geo#c.lyear, robust
predict hat_linfra_mobile, xb
replace linfra_mobile_ip = hat_linfra_mobile if linfra_mobile_ip == .

*1st round prediction internet
qui reg linfra_internet linfra_mobile_ip linfra_phoneline i.geo i.geo#c.lyear, robust
predict hat_linfra_internet, xb
replace linfra_internet_ip = hat_linfra_internet if linfra_internet_ip == .

*1st round prediction electricity
qui reg linfra_electricity linfra_internet_ip linfra_phoneline i.geo i.geo#c.lyear, robust  /* mobile not significant, hence dropped */
predict hat_linfra_electricity, xb
replace linfra_electricity_ip = hat_linfra_electricity if linfra_electricity_ip == .

*1st round prediction secureinternet
qui reg linfra_secureinternet linfra_electricity_ip i.geo i.geo#c.lyear, robust  /* only electricity significant, rest dropped */
predict hat_linfra_secureinternet, xb
replace linfra_secureinternet_ip = hat_linfra_secureinternet if linfra_secureinternet_ip == .

*1st round prediction phoneline
qui reg linfra_phoneline linfra_secureinternet_ip linfra_electricity_ip linfra_internet_ip linfra_mobile_ip i.geo i.geo#c.lyear, robust
predict hat_linfra_phoneline, xb
replace linfra_phoneline_ip = hat_linfra_phoneline if linfra_phoneline_ip == .

drop hat_*

*2nd round prediction mobile
qui reg linfra_mobile linfra_phoneline_ip i.geo i.geo#c.lyear, robust
predict hat_linfra_mobile, xb
replace linfra_mobile_ip = hat_linfra_mobile if linfra_mobile == .

*2nd round prediction internet
qui reg linfra_internet linfra_mobile_ip linfra_phoneline_ip i.geo i.geo#c.lyear, robust
predict hat_linfra_internet, xb
replace linfra_internet_ip = hat_linfra_internet if linfra_internet == .

*2nd round prediction electricity
qui reg linfra_electricity linfra_internet_ip linfra_phoneline_ip i.geo i.geo#c.lyear, robust  /* mobile not significant, hence dropped */
predict hat_linfra_electricity, xb
replace linfra_electricity_ip = hat_linfra_electricity if linfra_electricity == .

*2nd round prediction secureinternet
qui reg linfra_secureinternet linfra_electricity_ip i.geo i.geo#c.lyear, robust  /* only electricity significant, rest dropped */
predict hat_linfra_secureinternet, xb
replace linfra_secureinternet_ip = hat_linfra_secureinternet if linfra_secureinternet == .

*2nd round prediction phoneline
qui reg linfra_phoneline linfra_electricity_ip linfra_internet_ip linfra_mobile_ip i.geo i.geo#c.lyear, robust
predict hat_linfra_phoneline, xb
replace linfra_phoneline_ip = hat_linfra_phoneline if linfra_phoneline == .

gen dum_infraobs_complete = 0
replace dum_infraobs_complete = 1 if linfra_phoneline_ip!=. & linfra_mobile_ip!=. & linfra_internet_ip!=. & linfra_electricity_ip!=. & linfra_secureinternet_ip!=.

* Pearson standardization and weights for index

foreach var of varlist linfra_phoneline linfra_mobile linfra_internet linfra_electricity linfra_secureinternet {
qui sum `var'_ip if dum_infraobs_complete==1
gen pearson_`var' = (`var'_ip - r(mean))/r(sd) if dum_infraobs_complete==1

bys year: egen nrobs_`var' = count(`var') if dum_infraobs_complete==1
}

gen nrobs_infratotal = nrobs_linfra_phoneline + nrobs_linfra_mobile + nrobs_linfra_internet + nrobs_linfra_electricity + nrobs_linfra_secureinternet

foreach var of varlist linfra_phoneline linfra_mobile linfra_internet linfra_electricity linfra_secureinternet {
gen weight_`var' = nrobs_`var' / nrobs_infratotal
}

/* *control check:
gen sumofweights = weight_linfra_phoneline + weight_linfra_mobile + weight_linfra_internet + weight_linfra_electricity + weight_linfra_secureinternet
sum sumofweights
*/

* generate final index

gen infrastructure_index = weight_linfra_phoneline*pearson_linfra_phoneline + weight_linfra_mobile*pearson_linfra_mobile ///
 + weight_linfra_internet*pearson_linfra_internet + weight_linfra_electricity*pearson_linfra_electricity + weight_linfra_secureinternet*pearson_linfra_secureinternet
 
sum infrastructure_index

pca pearson*
predict infra_pca1 infra_pca2
